﻿local E = AleaUI
local L = E.L
local bubbles = E:Module("ChatBubbles")
local chatframe = E:Module("ChatFrames")
local MY_CLOSE_BUTTON = "×"
local strsplit = strsplit

local defaults = {
	enableTimeStamps = true,
	width = 384,
	height = 140,
	
	xOffset = 7,
	yOffset = 31,
	
	editBoxxOffset = 0,
	editBoxyOffset = 0,
	
	OffsetTop = 5,
	OffsetRight = 3,
	OffsetBottom = 6,
	OffsetLeft = 3,
	
	enable_background = true,
	disable_chatlog = false,
	enable_soundAlert = true,
	history = true,
	keyHandler = true,
	nameFilter = {},
	battleTagFilter = {},
	msgFilter = {		
		["GolDDeaL.ru"]  = { enable = true },
		["LvL-MoNeY.ru"] = { enable = true },
		['PLazaGame.RU'] = { enable = true },
	},
	["border"] = {
		["background_texture"] = AleaUI.media.default_bar_texture_name3,
		["size"] = 1,
		["inset"] = -1,
		["color"] = {
			0,  
			0,  
			0,  
			1,  
		},
		["background_inset"] = 0,
		["background_color"] = {
			0,  
			0,  
			0,  
			0.8,  
		},
		["texture"] = AleaUI.media.default_bar_texture_name3,
	},
}

E.default_settings.chatPanel = defaults

local wowString = BNET_CLIENT_WOW
local scString = BNET_CLIENT_SC2
local d3String = BNET_CLIENT_D3
local wtcgString = BNET_CLIENT_WTCG
local appString = BNET_CLIENT_APP
local overwatchString = BNET_CLIENT_OVERWATCH
local hotsString = BNET_CLIENT_HEROES

local decodeURI
do
   local char, gsub, tonumber = string.char, string.gsub, tonumber
   local function decode(hex) return char(tonumber(hex, 16)) end
   
   function decodeURI(s)
      s = gsub(s, '%%(%x%x)', decode)
      return s
   end
end

do
	local function Custom_FCF_UpdateButtonSide(chat)
		local leftDist =  chat:GetLeft() or 0;
		
		local rightDist = GetScreenWidth() - ( chat:GetRight() or 0 );
		local changed = nil;
		if (( leftDist > 0 and leftDist <= rightDist ) or rightDist < 0 ) then
			if ( chat.buttonSide ~= "left" ) then
				FCF_SetButtonSide(chat, "left");
				changed = 1;
			end
		else
			if ( chat.buttonSide ~= "right" or leftDist < 0 ) then
				FCF_SetButtonSide(chat, "right");
				changed = 1;
			end
		end
		return changed;
	end
	
	--FCF_UpdateButtonSide = Custom_FCF_UpdateButtonSide
end

do
	local enableTimeStamps = true
	local SELECTED_CHAT_FRAME_ALEA = ChatFrame1
	local hooks = {}
	
	local filterType = nil
	local filterSelectedToken = nil

	local function GetBattleNetFilterStatus(battleTag, game)
		
		local d = E.db.chatPanel.battleTagFilter[battleTag]

		if d and d.enable then
			
			if d.game1 and game == wowString then return true end
			if d.game2 and game == d3String then return true end
			if d.game3 and game == scString then return true end
			if d.game4 and game == wtcgString then return true end
			if d.game5 and game == overwatchString then return true end
			if d.game6 and game == hotsString then return true end
			if d.game7 and game == appString then return true end
			
		end
		
		
		return false
	end
	
	local SoundSys = CreateFrame("Frame")
	SoundSys:RegisterEvent("CHAT_MSG_WHISPER")
	SoundSys:RegisterEvent("CHAT_MSG_BN_WHISPER")
	SoundSys.spam = GetTime()
	SoundSys:HookScript("OnEvent", function(self, event, msg, author, ...)
		for frame in pairs(hooks) do
			frame.tellTimer = nil
		end
		
		if event == 'CHAT_MSG_BN_WHISPER' then
			local senderID = select(11, ...)
			local _, _, battleTag,_, _, _, client = BNGetFriendInfoByID(senderID)
		
			if GetBattleNetFilterStatus(battleTag, client) then
				return true
			end
		end
		
		if self.spam > GetTime() then return end
		self.spam = GetTime() + 3
		
		if E.db.chatPanel.enable_soundAlert then PlaySoundFile([[Interface\AddOns\AleaUI\media\whisper.mp3]],"Master") end
	end)
	
	ChatFrame_AddMessageEventFilter("CHAT_MSG_BN_WHISPER",function(f, event, msg, author, ...)
		local senderID = select(11, ...)
		local _, _, battleTag,_, _, _, client = BNGetFriendInfoByID(senderID)
		
		if GetBattleNetFilterStatus(battleTag, client) then
				return true
			end
	
		return false
	end)
	
	ChatFrame_AddMessageEventFilter("CHAT_MSG_BN_WHISPER_INFORM",function(f, event, msg, author, ...)
		
		local senderID = select(11, ...)
		local _, _, battleTag,_, _, _, client = BNGetFriendInfoByID(senderID)
	
		if GetBattleNetFilterStatus(battleTag, client) then
			return true
		end
	
		return false
	end)

	
	local copy_w, copy_h = 500, 300
	
	local PLAYER_REALM = gsub(E.myrealm,'[%s%-]','')
	local PLAYER_NAME = E.myname.."-"..PLAYER_REALM
	
	local backdrop = {
		bgFile = [[Interface\Buttons\WHITE8x8]], 
		edgeFile = [[Interface\Buttons\WHITE8x8]], 
		edgeSize = 1, 
	}
	
	chatframe.chatFrameWidth = 384
	chatframe.chatFrameHeight = 140
	chatframe.chatFrameXOffset = 7
	chatframe.chatFrameYOffset = 31
	
	local copyframe = CreateFrame("Frame", "AleaUIChatHistoryFrame", E.UIParent)
	copyframe:SetPoint("CENTER")
	copyframe:SetSize(copy_w, copy_h)
	copyframe:SetFrameLevel(15)
	
	local f = CreateFrame("Frame", nil, copyframe)
	f:SetPoint('CENTER', copyframe, 'CENTER', 0, 0)
	f:SetSize(copy_w, copy_h)
	f:SetFrameLevel(copyframe:GetFrameLevel() + 3)
	
	local resizeTabs = true
	local minTabWidth = 60
	
	tinsert(UISpecialFrames, "AleaUIChatHistoryFrame")
	
	copyframe.Scroll = CreateFrame("ScrollFrame", "AleaUIChatScrollFrame", copyframe, "UIPanelScrollFrameTemplate")
	copyframe.Scroll.ScrollBar:SetParent(f)
	
	copyframe.Scroll.ScrollBar:SetScript('OnValueChanged', function(self, value)
		copyframe.Scroll:SetVerticalScroll(value);
	end)
	
	copyframe.Scroll:SetFrameLevel(copyframe:GetFrameLevel() + 1)
	copyframe.Scroll:SetSize(copy_w, copy_h)
	copyframe.Scroll:SetPoint("TOPRIGHT", copyframe, "TOPRIGHT", 0, -2)
	
	copyframe.editBox = CreateFrame("EditBox", nil, E.UIParent)
	copyframe.editBox:SetPoint('TOPLEFT', copyframe.Scroll, "TOPLEFT", 11, 1)
	copyframe.editBox:SetFontObject(GameFontWhite)
	copyframe.Scroll:SetScrollChild(copyframe.editBox)
	copyframe.Scroll:SetHorizontalScroll(-5)
	copyframe.Scroll:SetVerticalScroll(0)
	copyframe.Scroll:EnableMouse(true)	
	copyframe.Scroll:SetClipsChildren(true)
	
	copyframe.editBox:SetSize(copy_w, copy_h)
	copyframe.editBox:SetFont("Fonts\\ARIALN.TTF", 12)
	copyframe.editBox:SetFrameLevel(copyframe.Scroll:GetFrameLevel() + 1)
	copyframe.editBox:SetAutoFocus(false)
	copyframe.editBox:SetMultiLine(true)
	copyframe.editBox:SetCountInvisibleLetters(false)
	copyframe.editBox:SetScript("OnEscapePressed", function(self)
		self:ClearFocus()
	end)
	copyframe:Hide()
	copyframe:SetBackdrop(backdrop)
	copyframe:SetBackdropColor(0, 0, 0, 0.8) --цвет фона
	copyframe:SetBackdropBorderColor(0 , 0 , 0 , 1) --цвет фона
	
	copyframe.close = CreateFrame("Button", nil, copyframe.editBox)
	copyframe.close:SetNormalTexture("Interface\\Buttons\\UI-Panel-MinimizeButton-Up")
	copyframe.close:SetPushedTexture("Interface\\Buttons\\UI-Panel-MinimizeButton-Down")
	copyframe.close:SetHighlightTexture("Interface\\Buttons\\UI-Panel-MinimizeButton-Highlight", "ADD")
	copyframe.close:SetSize(32, 32)
	copyframe.close:SetPoint('TOPRIGHT', copyframe, 'TOPRIGHT', 0, 0)
	copyframe.close:SetScript("OnClick", function(self) copyframe:Hide();copyframe.editBox:SetText(""); copyframe.editBox:ClearFocus()end)
	
	local function GetBattleTagName(id)
		local _, _, battleTag = BNGetFriendInfoByID(id)
		local name = battleTag and strsplit('#', battleTag)
		return name
	end
	
	local function GetBattleTagNameF(id)
	
		return 'BNetF:'..id
	end
	
	local function Sklonenie(a1, a2, a3, a4)
	   --  print('a1:',a1, 'a2:', a2, 'a3:', a3, 'a4:', a4)
	   if a1 == '1' then
		  return a1..' '..a2
	   elseif a1 == '2' or a1 == '3' or a1 == '4' then
		  return a1..' '..a3
	   else
		  return a1..' '..a4
	   end 
	end
				
	local function GetChatframeText()
		if SELECTED_CHAT_FRAME_ALEA then
			local msg = ""
			local start = 1
			local _max = SELECTED_CHAT_FRAME_ALEA:GetMaxLines()
			local _cur = SELECTED_CHAT_FRAME_ALEA:GetNumMessages()
			
			if _cur > _max then
				start = _cur - _max
			end
			
			for i=start, _cur do
				copyframe:Show()
				
				local msg2 = SELECTED_CHAT_FRAME_ALEA:GetMessageInfo(i)
				
				msg2 = msg2:gsub('|c%x%x%x%x%x%x%x%x', '')
				msg2 = msg2:gsub('|r', '')
				
				--[==[
				msg2 = msg2:gsub("|Hplayer:.-|h([^:]+)|h", '%1')
				msg2 = msg2:gsub("|Hachievement:.-|h([^:]+)|h", '%1')
				msg2 = msg2:gsub("|Hshareachieve:.-|h.-|t|h", '')
				msg2 = msg2:gsub("|Hitem:.-|h([^:]+)|h", '%1')
				msg2 = msg2:gsub("|Hchannel:.-|h([^:]+)|h", '%1')
				msg2 = msg2:gsub("|Hurl.-|h([^:]+)", '%1')
				msg2 = msg2:gsub("|HBNplayer:.-|h([^:]+)", '%1')
				msg2 = msg2:gsub("|Hcurrency:.-|h([^:]+)|h", '%1')			
				msg2 = msg2:gsub("|Htrade:.-|h([^:]+)|h", '%1')		
				msg2 = msg2:gsub("|Hquest:.-|h([^:]+)|h", '%1')
				msg2 = msg2:gsub("|Hlevelup:.-|h([^:]+)|h", '%1')
				msg2 = msg2:gsub("|Hachievement:.-|h([.+]+)|h", '%1')
				msg2 = msg2:gsub("|HbattlePetAbil.-|h([.+]+)|h", '%1')
				]==]
				
				msg2 = msg2:gsub("|Hshareachieve:.-|h.-|t|h", '')
				msg2 = msg2:gsub("|H.-:.-|h([^:]+)|h", '%1')				
				msg2 = msg2:gsub("|T.-|t", '')
				msg2 = msg2:gsub(' |%d.-%d%((.-)%)', ' %1')
				
				msg2 = msg2:gsub('(%d) |4(.-):(.-):(.-);', Sklonenie)
			
			--	msg2 = msg2:gsub("|Kb(.+)|k.-|k", GetBattleTagName)
			--	msg2 = msg2:gsub("|Kf(.+)|k.-|k", GetBattleTagNameF)
				
				--19:50:00 [|Kf94|k00000|k] (Exila) входит в сеть.
				
			--	msg2 = msg2:gsub("|Hachievement:.-|h([^:]+)|h", '%1')
			
			--	15:10:39 Получено: 11 |4золотая:золотые:золотых;, 40 |4серебряная:серебряные:серебряных;.
			--	15:10:39 Вы получаете 19180 |4очко:очка:очков; опыта.


				--  [|Kb21|k0000000000|k] 
				--  |Kb1994|k000000000000|k
				-- |Kf38 BNGetFriendInfoByID(38)
				
			--	msg2 = msg2:gsub('|h', '')
			--	msg2 = msg2:gsub("|", "||")
			--	msg2 = msg2:gsub("|c", "")
			--	msg2 = msg2:gsub("|r", "")
			--	msg2 = msg2:gsub("|T", '')
			--	msg2 = msg2:gsub("|t", '')
		
				msg = msg..msg2.."\n"				
			end
		
			copyframe.editBox:SetText(msg)
		end
	end
	
	local function ChatFrame_OnMouseScroll(frame, delta)
		if delta < 0 then
			if IsShiftKeyDown() then
				frame:ScrollToBottom()
			else
				for i = 1, 3 do
					frame:ScrollDown()
				end
			end
		elseif delta > 0 then
			if IsShiftKeyDown() then
				frame:ScrollToTop()
			else
				for i = 1, 3 do
					frame:ScrollUp()
				end
			end
		end
	end

	local function addcopybutton(frame)	
		local bttn = CreateFrame("Button",nil, E.UIParent)
		bttn:SetWidth(18)
		bttn:SetHeight(18)
		bttn:SetScale(1)
		bttn:SetAlpha(0.3)
		bttn:SetPoint('TOPRIGHT', frame, 'TOPRIGHT', 0, 0)
		bttn:SetNormalTexture("Interface\\GossipFrame\\PetitionGossipIcon.blp")
		bttn:GetNormalTexture():SetTexCoord(unpack(E.media.texCoord))
		bttn:SetScript("OnClick", function(self) GetChatframeText() end)
		bttn:SetMovable(true)
		bttn:SetUserPlaced(true)
		bttn:EnableMouse(true)
		bttn:RegisterForDrag("LeftButton","RightButton")
		bttn:SetHighlightTexture("Interface\\Tooltips\\UI-Tooltip-Background")	
	end
	
	local function HideForever(f)
		f:SetScript("OnShow", f.Hide)
		f:Hide()
	end

	local RAID_LEADER = RAID_LEADER
	local MY_CHAT_WHISPER_INFORM_GET = strsplit("\124", CHAT_WHISPER_INFORM_GET)
	local MY_CHAT_WHISPER_GET = string.match(CHAT_WHISPER_GET, ' ([%w%W]+):')
	local SAY_PATTERN = CHAT_SAY_GET:gsub(":", ''):gsub('%%s', ''):gsub(' ', '')
	local YELL_PATTERN = CHAT_YELL_GET:gsub(":", ''):gsub('%%s', ''):gsub(' ', '')
	
	local function customGSUBHandler(pre, sender, post)
		local name, server = strsplit("-", sender)		
		if server then
			local colorp = ( server:find('|r') ) and '|r' or ''
			name = name..'*'..colorp
		end	
		return pre..name..post
	end
	
	local function customUrlDecode(url)
		local decoded = decodeURI(url)
		return format('|cffffffff|Hurl~%s|h[%s]|h|r', decoded, decoded)
	end
	
	local function customKeyHandler(...)
		
		print('Tprint', 'customKeyHandler', ...)
		
		return 'ЭК'
	end
	
	local keyFindLastName = nil
	local function customGSUBHandler1(pre, itemname, post)
		return pre..keyFindLastName..post
	end
	
	local function AddMessage(frame, text, r, g, b, id, holdTime)
		
		local editMessage = true
		
		if text:find('Tprint') then
			editMessage = false
		end
		
		--[[	
			RAID

			GUILD

			LEADER

			PARTY_LEADER
			RAID_LEADER

			GUIDE
			
			CHAT_WHISPER_INFORM_GET
			
			
			INSTANCE_CHAT_LEADER

			CHAT_MESSAGE_RAID_LEADER

			OFFICER

			INSTANCE_CHAT

			INSTANCE

			CHAT_WHISPER_GET
		]]
		if(editMessage) then
			text = text:gsub("%["..PARTY_LEADER.."%]", "[ЛГр]")
			text = text:gsub("%["..GUILD.."%]", "[Г]")
			text = text:gsub("%["..PARTY.."%]", "[Гр]")
			text = text:gsub("%["..RAID.."%]", "[Р]")
			text = text:gsub("%["..RAID_LEADER.."%]", "[ЛР]")
			text = text:gsub("^%["..RAID_WARNING.."%]", "[ОР]")		
			text = text:gsub("%["..OFFICER.."%]", "[O]")
			text = text:gsub("%["..INSTANCE.."%]", "[П]")
			text = text:gsub("<"..AFK..">", "[|cffFF0000AFK|r] ")
			text = text:gsub("<"..DND..">", "[|cffE7E716DND|r] ")
			text = text:gsub("%["..INSTANCE_CHAT_LEADER.."%]", "[ЛП]")
			text = text:gsub("%[(%d0?)%. .-%]", "[%1]") -- Обрезает имена каналов до цифры
			text = text:gsub("^(.-|h) "..MY_CHAT_WHISPER_GET, "%1 >>")
			text = text:gsub(MY_CHAT_WHISPER_INFORM_GET.."(.-|h)", ">> %1")
		
			--text = text:gsub("^(.-|h) говорит", "[С] %1")
			text = text:gsub("^(.-|h) "..SAY_PATTERN, "[С] %1")
		
			text = text:gsub("^(.-|h) "..YELL_PATTERN, "[К] %1")
			text = text:gsub('([wWhH][wWtT][wWtT][%.pP]%S+[^%p%s])', customUrlDecode) --'|cffffffff|Hurl~%1|h[%1]|h|r')
			
			
		--	local namePattern, findName, findServer = createPattern(text)
		
		--	print("Tprint", namePattern, findName, findServer)
			
		--	if findName and findServer then		
			--	text = text:gsub(namePattern, gsubNamePattern)
		--		print("Tprint", namePattern, findName, findServer)
		--	end
			
		--	local temp = text:gsub("(|Hplayer:.-|h%[)(.-)(%]|h)", customGSUBHandler)
			text = text:gsub("(|Hplayer:.-|h%[)(.-)(%]|h)", customGSUBHandler)
			
			if E.db and E.db.chatPanel.keyHandler and text:match('|Hitem:138019') then
			
				local affixes, dungeonID, keystoneLevel = text:match('|Hitem:138019:.-:.-:.-:.-:.-:.-:.-:.-:.-:(%d+):.-:.-:(%d+):(%d+):')
				local dungeonName
				local itemName = GetItemInfo(138019)
				
				dungeonID = tonumber(dungeonID)

				if dungeonID then
					dungeonName = C_ChallengeMode.GetMapInfo(dungeonID)				
					keyFindLastName = format(CHALLENGE_MODE_KEYSTONE_NAME, dungeonName..' +'..keystoneLevel)
				else
					keyFindLastName = itemName
				end

				text = text:gsub("(|Hitem:.-|h%[)(.-)(%]|h)", customGSUBHandler1)
			end
			
		--	print('Tprint', temp)
			
			
			if text:find('|Hshare') then
				text = text:gsub(':18:18|t', ':0:0|t')		
			end
		end
	
		if(enableTimeStamps) then
			text = "|cff999999" .. date("%H:%M:%S") .. "|r " .. text
		end

		return hooks[frame](frame, text, r, g, b, id, holdTime)
	end

	local tags2 = {"ButtonFrameUpButton", "ButtonFrameDownButton", "ButtonFrameBottomButton", "ConversationButton", "ButtonFrameMinimizeButton"}
	local function updateGameChatButtons(frame)	
		for i, name in ipairs(tags2) do
			
			local f = _G[frame:GetName()..name]
			
			if f and not f.tx then
	
				f:GetNormalTexture():SetDesaturated(true)
				f:GetNormalTexture():SetTexCoord(0.17, 0.77, 0.23, 0.78)
				f:GetNormalTexture():ClearAllPoints()
				f:GetNormalTexture():SetPoint("TOPLEFT", f,"TOPLEFT", 3,-3)
				f:GetNormalTexture():SetPoint("BOTTOMRIGHT", f,"BOTTOMRIGHT",  -3,3)

				f:GetPushedTexture():SetDesaturated(true)
				f:GetPushedTexture():SetTexCoord(0.17, 0.77, 0.23, 0.78)
				f:GetPushedTexture():ClearAllPoints()
				f:GetPushedTexture():SetPoint("TOPLEFT", f,"TOPLEFT", 3,-3)
				f:GetPushedTexture():SetPoint("BOTTOMRIGHT", f,"BOTTOMRIGHT",  -3,3)

				f.tx = f.tx or f:CreateTexture(nil, "BORDER", nil , -1)
				f.tx:SetColorTexture(0, 0, 0, 1)
				f.tx:SetPoint("TOPLEFT", f,"TOPLEFT", 2,-2)
				f.tx:SetPoint("BOTTOMRIGHT", f,"BOTTOMRIGHT", -2,2)

			end
		end
	end
	
	local function SelectChatFrame(f, button, arg)
		if button == "LeftButton" and arg == false then			
			local id = tonumber(string.match(f:GetName(), "ChatFrame(%d+)Tab"))
			SELECTED_CHAT_FRAME_ALEA = _G[("ChatFrame%d"):format(id)]
			
			updateGameChatButtons(SELECTED_CHAT_FRAME_ALEA)
		end
	end
	
	local hookedEditBox = {}
	
	local leftDef, parentDef, rightDef = "BOTTOMLEFT", E.UIParent, "BOTTOMLEFT"
	local editXoffset, editYoffset = nil, nil
	
	local updatePosition = CreateFrame("Frame")
	updatePosition.elapsed = 0
	updatePosition:SetScript("OnUpdate", function(self, elapsed)
		self.elapsed = self.elapsed + elapsed
		if self.elapsed < 0.5 then return end
		self.elapsed = 0
		
		if not E.db.chatPanel then return end
		
		local left, parent, right, xoffset, yoffset = ChatFrame1:GetPoint()
		
		if left ~= leftDef or right ~= rightDef or xoffset ~= E.db.chatPanel.xOffset or yoffsetDef ~= E.db.chatPanel.yOffset then
			
	
			ChatFrame1:ClearAllPoints()
			ChatFrame1:SetPoint(leftDef, parentDef, rightDef, E.db.chatPanel.xOffset, E.db.chatPanel.yOffset)
			ChatFrame1:SetHeight(E.db.chatPanel.height)
			ChatFrame1:SetWidth(E.db.chatPanel.width)
			
		end
		
		if E.db.chatPanel.editBoxxOffset ~= editXoffset or
			E.db.chatPanel.editBoxyOffset ~= editYoffset then
			
			editXoffset = E.db.chatPanel.editBoxxOffset
			editYoffset = E.db.chatPanel.editBoxyOffset
			
		
			for editBox in pairs(hookedEditBox) do			
				editBox:ClearAllPoints()
				editBox:SetPoint("TOPLEFT", ChatFrame1, "BOTTOMLEFT", -2, E.db.chatPanel.editBoxyOffset)
				editBox:SetPoint("TOPRIGHT", ChatFrame1, "BOTTOMRIGHT",  2, E.db.chatPanel.editBoxyOffset)
			end
		end
	end)
	
	local tabs = {
		"Left", "Middle", "Right", "SelectedLeft", "SelectedMiddle",
		"SelectedRight", "HighlightLeft", "HighlightMiddle",
		"HighlightRight"
	}
	
	local bordersTex = {		
		"Background",
		'TopLeftTexture',
		'BottomLeftTexture',
		'TopRightTexture',
		'BottomRightTexture',
		'LeftTexture',
		'RightTexture',
		'BottomTexture',
		'TopTexture',
	}
	
	local tabSizeUpdate = function(self, a)
		if a ~= minTabWidth then
			
			if self.conversationIcon then
				self.conversationIcon:ClearAllPoints()
				self.conversationIcon:Kill()
			end

			self:GetFontString():ClearAllPoints()
			self:GetFontString():SetPoint("LEFT", self, "LEFT", 0, -5)
			self:GetFontString():SetJustifyH("CENTER")
			self:GetFontString():SetPoint("RIGHT", self, "RIGHT", 0, -5)
			self:SetWidth(minTabWidth)
					
			self.glow:ClearAllPoints()
			self.glow:SetPoint("LEFT", self, "LEFT", -2, -5)
			self.glow:SetPoint("RIGHT", self, "RIGHT", 2, -5)
		end
	end
	
	local customTabName = {
		["Общий"]   = "|cFFFFFFFF",
		['Каналы']  = "|cFFFFFFFF",
		['Гильдия'] = "|cFF00FF00",
		['Г']		= "|cFF00FF00",
		['Шепот']   = "|cFFe980e0",
		['Ш']   	= "|cFFe980e0",
		['Добыча']  = "|cFFFFFFFF",
		['Рейд']    = "|cFFFFFFFF",
		
		[GENERAL]	= "|cFFFFFFFF",
		[GUILD] 	= "|cFF00FF00",
		[CHANNELS]	= "|cFFFFFFFF",
		[WHISPER]	= "|cFFe980e0",
		[LOOT]		= "|cFFFFFFFF",
		['G']       = "|cFF00FF00",
		['W']       = "|cFFe980e0",
	}
	
	local tabTextUpdate = function(self, text)
	
		if self.conversationIcon then
			self.conversationIcon:ClearAllPoints()
			self.conversationIcon:Kill()
		end

		if customTabName[text] then
			self:GetFontString():SetText(customTabName[text]..text)
		else
			local name, server = strsplit("-", text or '')
			if server then			
				self:GetFontString():SetText(name)
			end
		end
	end

	local borderoffset = 1
	
	local function CustomChatFrame_MessageEventHandler(self, event, ...)
		
		return false
	end


	local function ChatFrame_Handler(...)
		if ( ChatFrame_ConfigEventHandler(...) ) then
			return;
		end
		if ( ChatFrame_SystemEventHandler(...) ) then
			return
		end
		if ( CustomChatFrame_MessageEventHandler(...) ) then
			return
		end
		if ( ChatFrame_MessageEventHandler(...) ) then
			return
		end
	end

	local function FloatingChatFrame_Handler(...)
		ChatFrame_Handler(...);
		FloatingChatFrame_OnEvent(...);
	end

	local function StyleChatFrame(name)
		local frame = _G[name]		
		if not frame or frame.aleauihooked then return end	
		frame.aleauihooked = true		
		updateGameChatButtons(frame)
	--	frame:SetIgnoreFramePositionManager(true)
		
		_G[name.."Tab"]:HookScript("OnClick", SelectChatFrame)
		_G[name.."Tab"]:GetFontString():SetFont(E.media.default_font, E.media.default_font_size-1, "OUTLINE")
		frame:SetClampRectInsets(0, 0, 0, 0)
		
		_G[name.."EditBoxLeft"]:SetAlpha(0)
		_G[name.."EditBoxRight"]:SetAlpha(0)
		_G[name.."EditBoxMid"]:SetAlpha(0)

		_G[name.."EditBoxFocusLeft"]:SetAlpha(0)
		_G[name.."EditBoxFocusRight"]:SetAlpha(0)
		_G[name.."EditBoxFocusMid"]:SetAlpha(0)				
		
		frame:SetFrameStrata("DIALOG")
		
		frame.editBox:ClearAllPoints()
		frame.editBox:SetPoint("TOPLEFT", ChatFrame1, "BOTTOMLEFT", -2, 0)
		frame.editBox:SetPoint("TOPRIGHT", ChatFrame1, "BOTTOMRIGHT",  2, 0)
		
		if not hookedEditBox[frame.editBox] then
			hookedEditBox[frame.editBox] = true
		end
		
		hooksecurefunc(frame.editBox, "SetFocus", frame.editBox.Show)
		hooksecurefunc(frame.editBox, "ClearFocus", frame.editBox.Hide)
	
		frame.editBox.border = CreateFrame("Frame", nil, frame.editBox)
		frame.editBox.border:SetFrameLevel(frame.editBox:GetFrameLevel()-1)
		frame.editBox.border:SetFrameStrata("LOW")
		frame.editBox.border:SetBackdrop(backdrop)
		frame.editBox.border:SetBackdropColor(0,0,0,0.9)
		frame.editBox.border:SetBackdropBorderColor(1,1,1,1)
		frame.editBox.border:SetPoint("LEFT", -borderoffset-1, 0)		
		frame.editBox.border:SetPoint("RIGHT", borderoffset, 0)
		frame.editBox.border:SetPoint("TOP", 0, -7)		
		frame.editBox.border:SetPoint("BOTTOM", 0, 5)
		
		frame.editBox:Hide()
		
		hooksecurefunc(_G[name.."EditBoxFocusRight"], "SetVertexColor", function(self, r,g,b,a)
			self.border:SetBackdropBorderColor(r,g,b,1)		
		end)
		
		hooksecurefunc(_G[name.."EditBoxFocusRight"], "Show", function(self, r,g,b,a)
			self.border:Show()
		end)
		
		hooksecurefunc(_G[name.."EditBoxFocusRight"], "Hide", function(self, r,g,b,a)
			self.border:Hide()
		end)
		
		_G[name.."EditBoxFocusRight"].border = frame.editBox.border
		
		--Enable moving in editbox without holding alt
		frame.editBox:SetAltArrowKeyMode(nil)

		for index, value in pairs(tabs) do
			local texture = _G[name.."Tab"..value]
			texture:SetTexture(nil)
		end
		
		for i=1, #bordersTex do	
			_G[name..bordersTex[i]]:SetTexture(nil)
			_G[name.."ButtonFrame"..bordersTex[i]]:SetTexture(nil)
		end

		tabTextUpdate(_G[name.."Tab"], _G[name.."Tab"]:GetFontString():GetText())
		
		if resizeTabs then
			hooksecurefunc(_G[name.."Tab"], "SetWidth", tabSizeUpdate)		
			_G[name.."Tab"]:SetWidth(1)			
		end
		
		hooksecurefunc(_G[name.."Tab"], "SetText", tabTextUpdate)
		hooksecurefunc(_G[name.."Tab"], "SetAlpha", function(t, alpha)			
			if alpha ~= 1 and ( GeneralDockManager.selected:GetID() == t:GetID() ) then
				t:SetAlpha(1)
			elseif alpha < 0.6 then
				t:SetAlpha(0.6)
			end
		end)
		if not frame.scriptsSet then
			frame:SetScript("OnMouseWheel", ChatFrame_OnMouseScroll)
			hooksecurefunc(frame, "SetScript", function(f, script, func)
				if script == "OnMouseWheel" and func ~= ChatFrame_OnMouseScroll then
					f:SetScript(script, ChatFrame_OnMouseScroll)
				end
			end)
			
			if frame:GetID() > NUM_CHAT_WINDOWS then
			--	frame:SetScript("OnEvent", FloatingChatFrame_Handler)
			elseif frame:GetID() ~= 2 then
			--	frame:SetScript("OnEvent", ChatFrame_Handler)
			end
			
			frame.scriptsSet = true
		end

		if frame:GetID() ~= 2 then
			hooks[frame] = frame.AddMessage
			frame.AddMessage = AddMessage
		end
	end
	
	chatframe.StyleChatFrame = StyleChatFrame
	
	local CHAT_FRAMES = CHAT_FRAMES
	local function checkForNewWindows()
	
		for _, chatFrameName in pairs(CHAT_FRAMES) do
		--	local frame = _G[chatFrameName];
			StyleChatFrame(chatFrameName)
		end
		
		--[==[
		for i=NUM_CHAT_WINDOWS+1, NUM_CHAT_WINDOWS+10 do
			StyleChatFrame(format("ChatFrame%d", i))
		end
		]==]
	end

	hooksecurefunc("FCF_OpenTemporaryWindow", checkForNewWindows)

	local chatCustomNameSenderReciever = {}
	
	local function CustomMessageHandler(self, event, a1,a2,a3,a4,a5,a6,a7,a8,a9,a10,a11,a12,a13,a14,a15,a16, ...)
	
		if event == "CHAT_MSG_WHISPER_INFORM" then
			local message = a1
			local receiver = a2
			
			local name, server = strsplit('-', receiver)		
		--	print("T", event, message, receiver)
			
			if name and not server then
				local newchatTarget = name.."-"..PLAYER_REALM				
				a2 = newchatTarget
		--		print("Erorr with server on", name)
			end
		elseif event == "CHAT_MSG_WHISPER" then
			local message = a1
			local sender = a2
			
			local name, server = strsplit('-', sender)
			
		--	print("T", event, message, sender)
			
			if name and not server then
				local newchatTarget = name.."-"..PLAYER_REALM				
				a2 = newchatTarget
		--		print("Erorr with server on", name)
			end
		end
		
		return self, event, a1,a2,a3,a4,a5,a6,a7,a8,a9,a10,a11,a12,a13,a14,a15,a16, ...
	end
	
	local function CustomFloatingChatFrameManager_OnEvent(...)
		FloatingChatFrameManager_OnEvent(CustomMessageHandler(...))
	end
	
	if false then
		QuickJoinToastButton:Kill()
	else
		QuickJoinToastButton:ClearAllPoints()
		QuickJoinToastButton:SetPoint('BOTTOMLEFT', ChatFrame1, 'TOPLEFT', -8, 30) 
	end
	
	for i = 1, NUM_CHAT_WINDOWS do
		StyleChatFrame(format("ChatFrame%d", i))		
		HideForever(_G[("ChatFrame%sButtonFrame"):format(i)])
		
		_G['ChatFrame'..i]:HookScript('OnShow', function(self)
			chatframe:SetLastBackgroundParent(self)
		end)
	end

	addcopybutton(_G["ChatFrame1"])
		
	ChatFrame1:ClearAllPoints()
	ChatFrame1:SetPoint("BOTTOMLEFT", E.UIParent, "BOTTOMLEFT", 7, 31)
	ChatFrame1:SetHeight(chatframe.chatFrameHeight)
	ChatFrame1:SetWidth(chatframe.chatFrameWidth)
	
	FCF_SavePositionAndDimensions(ChatFrame1)
	
	--Hide buttons
	ChatFrameMenuButton:Kill()
	
	--Make channels sticky
	ChatTypeInfo.SAY.sticky = 1
	ChatTypeInfo.EMOTE.sticky = 1
	ChatTypeInfo.YELL.sticky = 1
	ChatTypeInfo.PARTY.sticky = 1
	ChatTypeInfo.GUILD.sticky = 1
	ChatTypeInfo.OFFICER.sticky = 1
	ChatTypeInfo.RAID.sticky = 1
	ChatTypeInfo.RAID_WARNING.sticky = 1
	ChatTypeInfo.WHISPER.sticky = 1
	ChatTypeInfo.INSTANCE_CHAT.sticky = 1

	local SetHyperlink = ItemRefTooltip.SetHyperlink
	function ItemRefTooltip:SetHyperlink(data, ...)
		local isURL, _mycurlink = strsplit("~", data)
		if isURL and isURL == "url" then
			chatframe:Popup(_mycurlink)
		else
			SetHyperlink(self, data, ...)
		end
	end

	local defaultFilter =  "%[Game Master%] GM:"
	local _keyWords = {}
	local maxKeyWords = #_keyWords
	local function UpdateFilterMsgList()
		wipe(_keyWords)
		
		_keyWords[#_keyWords+1] = defaultFilter
		
		for k, v in pairs(E.db.chatPanel.msgFilter) do
			if v and v.enabled then
				_keyWords[#_keyWords+1] = k
			end
		end
		
		maxKeyWords = #_keyWords
	end

	local find = string.find
	local match = string.match
	local function spamFiler(f, event, msg, author)
		for i=1, maxKeyWords do
			if find(msg, _keyWords[i]) then	
				return true
			end			
		end	
		return false
	end
	
	
	ChatFrame_AddMessageEventFilter("CHAT_MSG_WHISPER",spamFiler)
	ChatFrame_AddMessageEventFilter("CHAT_MSG_CHANNEL",spamFiler)

	local chatFrameBackground = CreateFrame("Frame", nil, E.UIParent)
	chatFrameBackground:SetFrameStrata("BACKGROUND")
	chatFrameBackground:SetPoint("TOPLEFT", _G['ChatFrame1'], "TOPLEFT", -3, 5)
	chatFrameBackground:SetPoint("BOTTOMRIGHT", _G['ChatFrame1'], "BOTTOMRIGHT", 3, -6)
	
	local artBorder = CreateFrame("Frame", nil, chatFrameBackground)
	artBorder:SetBackdrop({
	  edgeFile = [[Interface\Buttons\WHITE8x8]],
	  edgeSize = 1, 
	})
	artBorder:SetBackdropBorderColor(0,0,0,1)
	artBorder:SetPoint("TOPLEFT", chatFrameBackground, "TOPLEFT", 0, -0)
	artBorder:SetPoint("BOTTOMRIGHT", chatFrameBackground, "BOTTOMRIGHT", -0, 0)

	artBorder.back = artBorder:CreateTexture()
	artBorder.back:SetDrawLayer('ARTWORK', -2)
	artBorder.back:SetColorTexture(0, 0, 0, 0)
	artBorder.back:SetPoint("TOPLEFT", chatFrameBackground, "TOPLEFT", 0, 0)
	artBorder.back:SetPoint("BOTTOMRIGHT", chatFrameBackground, "BOTTOMRIGHT", 0, 0)
	
	function chatframe:SetLastBackgroundParent(me)
	--	chatFrameBackground:SetParent(me)
	--	chatFrameBackground:SetFrameLevel(me:GetFrameLevel()-1)
	end
	
	function chatframe:UpdateChatBackground()		
		chatFrameBackground:SetPoint("TOPLEFT", _G['ChatFrame1'], "TOPLEFT", -E.db.chatPanel.OffsetLeft, E.db.chatPanel.OffsetTop)
		chatFrameBackground:SetPoint("BOTTOMRIGHT", _G['ChatFrame1'], "BOTTOMRIGHT", E.db.chatPanel.OffsetRight, -E.db.chatPanel.OffsetBottom)	
		
		local opts = E.db.chatPanel
		
		artBorder:SetBackdrop({
		  edgeFile = E:GetBorder(opts.border.texture),
		  edgeSize = opts.border.size, 
		})
		artBorder:SetBackdropBorderColor(opts.border.color[1],opts.border.color[2],opts.border.color[3],opts.border.color[4])
		artBorder:SetPoint("TOPLEFT", chatFrameBackground, "TOPLEFT", opts.border.inset, -opts.border.inset)
		artBorder:SetPoint("BOTTOMRIGHT", chatFrameBackground, "BOTTOMRIGHT", -opts.border.inset, opts.border.inset)

		artBorder.back:SetTexture(E:GetTexture(opts.border.background_texture))
		artBorder.back:SetVertexColor(opts.border.background_color[1],opts.border.background_color[2],opts.border.background_color[3],opts.border.background_color[4])
		artBorder.back:SetPoint("TOPLEFT", chatFrameBackground, "TOPLEFT", opts.border.background_inset, -opts.border.background_inset)
		artBorder.back:SetPoint("BOTTOMRIGHT", chatFrameBackground, "BOTTOMRIGHT", -opts.border.background_inset, opts.border.background_inset)
	end
	
	GeneralDockManagerOverflowButton:ClearAllPoints()
	GeneralDockManagerOverflowButton:SetPoint('BOTTOMRIGHT', _G['ChatFrame1'], 'TOPRIGHT', 2, 7)
	hooksecurefunc(GeneralDockManagerScrollFrame, 'SetPoint', function(self, point, anchor, attachTo, x, y)
		if anchor == GeneralDockManagerOverflowButton and x == 0 and y == 0 then
			self:SetPoint(point, anchor, attachTo, -2, -6)
		end
	end)
	
	GeneralDockManagerOverflowButtonList:StripTextures()
	E:CreateBackdrop(GeneralDockManagerOverflowButtonList, GeneralDockManagerOverflowButtonList, { 0, 0, 0, 1 }, { 0, 0, 0, 0.6 }, "BACKGROUND")
	
	E.GUI.args.ChatPanel = {
		name = L['Chat'],
		type = "group",
		order = 5,
		expand = true,
		args = {},
	}
	
	E.GUI.args.ChatPanel.args.Blacklist = {
		name = L['Filters'],
		type = "group",
		order = 5,
		embend = false,
		args = {},
	}
	
	E.GUI.args.ChatPanel.args.Blacklist.args.filterType = {	
		name = L['Select filter'],
		order = 1,
		type = "dropdown",
		width = 'full',
		values = {			
			L['Black list - by name'],
			L['Black list - by battleTag'],
			L['Black list - by message'],
		},
		set = function(self, value)
			filterType = value
			filterSelectedToken = nil
			chatframe:UpdateFilterGUI(value)
		end,
		get = function(self)
			return filterType
		end,
	}
	
	E.GUI.args.ChatPanel.args.keyHandler = {		
		name = L['Enable more key link info'],
		type = 'toggle', width = 'full',
		order = 0.1,
		set = function()
			E.db.chatPanel.keyHandler = not E.db.chatPanel.keyHandler
		end,
		get = function()
			return E.db.chatPanel.keyHandler
		end,	
	}
	
	E.GUI.args.ChatPanel.args.HideBackground = {		
		name = L['Enable background'],
		type = 'toggle',
		order = 1,
		set = function()
			E.db.chatPanel.enable_background = not E.db.chatPanel.enable_background
			
			if E.db.chatPanel.enable_background then
				chatFrameBackground:Show()
			else
				chatFrameBackground:Hide()
			end
		end,
		get = function()
			return E.db.chatPanel.enable_background
		end,	
	}
	
	E.GUI.args.ChatPanel.args.RemoveLog = {		
		name = L['Hide combat log tab'],
		type = 'toggle',
		order = 2,
		set = function()
			E.db.chatPanel.disable_chatlog = not E.db.chatPanel.disable_chatlog		
			chatframe:UpdateCombatLogChatSettings()
		end,
		get = function()
			return E.db.chatPanel.disable_chatlog
		end,	
	}
	
	E.GUI.args.ChatPanel.args.SoundAlert = {		
		name = L['Enable sound alert'],
		type = 'toggle',
		order = 2.2,
		set = function()
			E.db.chatPanel.enable_soundAlert = not E.db.chatPanel.enable_soundAlert
		end,
		get = function()
			return E.db.chatPanel.enable_soundAlert
		end,	
	}
	
	E.GUI.args.ChatPanel.args.History = {		
		name = L['Enable message history'],
		type = 'toggle',
		order = 2.3,
		set = function()
			E.db.chatPanel.history = not E.db.chatPanel.history		
			chatframe:ToggleChatHistory()
		end,
		get = function()
			return E.db.chatPanel.history
		end,	
	}

	E.GUI.args.ChatPanel.args.Width = {
		name = L['Width'],
		order = 3,
		type = 'slider',
		min= 140, max = 900, step = 1,
		set = function(info, value)
			E.db.chatPanel.width = value
			updatePosition.elapsed = 0
		end,
		get = function(info)
			return E.db.chatPanel.width
		end,
	}
	
	E.GUI.args.ChatPanel.args.height = {
		name = L['Height'],
		order = 4,
		type = 'slider',
		min= 100, max = 600, step = 1,
		set = function(info, value)
			E.db.chatPanel.height = value
			updatePosition.elapsed = 0
		end,
		get = function(info)
			return E.db.chatPanel.height
		end,
	}

	E.GUI.args.ChatPanel.args.xOffset = {
		name = L['Horizontal offset'],
		order = 4.1,
		type = 'slider',
		min= 0, max = 2000, step = 1,
		set = function(info, value)
			E.db.chatPanel.xOffset = value
			updatePosition.elapsed = 0
		end,
		get = function(info)
			return E.db.chatPanel.xOffset
		end,
	}
	
	E.GUI.args.ChatPanel.args.yOffset = {
		name = L['Vertical offset'],
		order = 4.2,
		type = 'slider',
		min= 0, max = 2000, step = 1,
		set = function(info, value)
			E.db.chatPanel.yOffset = value
			updatePosition.elapsed = 0
		end,
		get = function(info)
			return E.db.chatPanel.yOffset
		end,
	}
	
	E.GUI.args.ChatPanel.args.editBoxxOffset = {
		name = L['Editbox horizontal offset'],
		order = 4.3,
		type = 'slider',
		min= 0, max = 2000, step = 1,
		set = function(info, value)
			E.db.chatPanel.editBoxxOffset = value
			updatePosition.elapsed = 0
		end,
		get = function(info)
			return E.db.chatPanel.editBoxxOffset
		end,
	}
	
	E.GUI.args.ChatPanel.args.editBoxyOffset = {
		name = L['Editbox vertical offset'],
		order = 4.4,
		type = 'slider',
		min= 0, max = 2000, step = 1,
		set = function(info, value)
			E.db.chatPanel.editBoxyOffset = value
			updatePosition.elapsed = 0
		end,
		get = function(info)
			return E.db.chatPanel.editBoxyOffset
		end,
	}
	
	
	
	E.GUI.args.ChatPanel.args.Background = {
		name = L['Background'],
		type = "group",
		order = 5,
		embend = false,
		args = {},
	}
	--[==[
	E.GUI.args.ChatPanel.args.Background.args.Enable = {		
		name = 'Включить',
		type = 'toggle',
		width = 'full',
		order = 1,
		set = function()
			
		end,
		get = function()
			return false
		end,	
	}
	]==]
	E.GUI.args.ChatPanel.args.Background.args.OffsetLeft = {
		name = L['Offset form left'],
		order = 2,
		type = 'slider',
		min= 0, max = 50, step = 1,
		set = function(info, value)
			E.db.chatPanel.OffsetLeft = value
			chatframe:UpdateChatBackground()	
		end,
		get = function(info)
			return E.db.chatPanel.OffsetLeft
		end,
	}
	
	E.GUI.args.ChatPanel.args.Background.args.OffsetRight = {
		name = L['Offset form right'],
		order = 2.1,
		type = 'slider',
		min= 0, max = 50, step = 1,
		set = function(info, value)
			E.db.chatPanel.OffsetRight = value
			chatframe:UpdateChatBackground()	
		end,
		get = function(info)
			return E.db.chatPanel.OffsetRight
		end,
	}
	
	E.GUI.args.ChatPanel.args.Background.args.OffsetTop = {
		name = L['Offset form top'],
		order = 2.2,
		type = 'slider',
		min= 0, max = 50, step = 1,
		set = function(info, value)
			E.db.chatPanel.OffsetTop = value
			chatframe:UpdateChatBackground()	
		end,
		get = function(info)
			return E.db.chatPanel.OffsetTop
		end,
	}
	
	E.GUI.args.ChatPanel.args.Background.args.OffsetBottom = {
		name = L['Offset form bottom'],
		order = 2.3,
		type = 'slider',
		min= 0, max = 50, step = 1,
		set = function(info, value)
			E.db.chatPanel.OffsetBottom = value
			chatframe:UpdateChatBackground()	
		end,
		get = function(info)
			return E.db.chatPanel.OffsetBottom
		end,
	}
	
	E.GUI.args.ChatPanel.args.Background.args.BorderOpts = {
		name = L['Borders'],
		order = 10,
		embend = true,
		type = "group",
		args = {}
	}
	
	E.GUI.args.ChatPanel.args.Background.args.BorderOpts.args.BorderTexture = {
		order = 1,
		type = 'border',
		name = L['Border texture'],
		values = E:GetBorderList(),
		set = function(info,value) 
			E.db.chatPanel.border.texture = value;
			chatframe:UpdateChatBackground()
		end,
		get = function(info) return E.db.chatPanel.border.texture end,
	}

	E.GUI.args.ChatPanel.args.Background.args.BorderOpts.args.BorderColor = {
		order = 2,
		name = L['Border color'],
		type = "color", 
		hasAlpha = true,
		set = function(info,r,g,b,a) 
			E.db.chatPanel.border.color={ r, g, b, a}; 
			chatframe:UpdateChatBackground()
		end,
		get = function(info) 
			return E.db.chatPanel.border.color[1],
					E.db.chatPanel.border.color[2],
					E.db.chatPanel.border.color[3],
					E.db.chatPanel.border.color[4] 
		end,
	}

	E.GUI.args.ChatPanel.args.Background.args.BorderOpts.args.BorderSize = {
		name = L['Border size'],
		type = "slider",
		order	= 3,
		min		= 1,
		max		= 32,
		step	= 1,
		set = function(info,val) 
			E.db.chatPanel.border.size = val
			chatframe:UpdateChatBackground()
		end,
		get =function(info)
			return E.db.chatPanel.border.size
		end,
	}

	E.GUI.args.ChatPanel.args.Background.args.BorderOpts.args.BorderInset = {
		name = L['Border inset'],
		type = "slider",
		order	= 4,
		min		= -32,
		max		= 32,
		step	= 1,
		set = function(info,val) 
			E.db.chatPanel.border.inset = val
			chatframe:UpdateChatBackground()
		end,
		get =function(info)
			return E.db.chatPanel.border.inset
		end,
	}


	E.GUI.args.ChatPanel.args.Background.args.BorderOpts.args.BackgroundTexture = {
		order = 5,
		type = 'statusbar',
		name = L['Background texture'],
		values = E.GetTextureList,
		set = function(info,value) 
			E.db.chatPanel.border.background_texture = value;
			chatframe:UpdateChatBackground()
		end,
		get = function(info) return E.db.chatPanel.border.background_texture end,
	}

	E.GUI.args.ChatPanel.args.Background.args.BorderOpts.args.BackgroundColor = {
		order = 6,
		name = L['Background color'],
		type = "color", 
		hasAlpha = true,
		set = function(info,r,g,b,a) 
			E.db.chatPanel.border.background_color={ r, g, b, a}
			chatframe:UpdateChatBackground()
		end,
		get = function(info) 
			return E.db.chatPanel.border.background_color[1],
					E.db.chatPanel.border.background_color[2],
					E.db.chatPanel.border.background_color[3],
					E.db.chatPanel.border.background_color[4] 
		end,
	}


	E.GUI.args.ChatPanel.args.Background.args.BorderOpts.args.backgroundInset = {
		name = L['Background inset'],
		type = "slider",
		order	= 7,
		min		= -32,
		max		= 32,
		step	= 1,
		set = function(info,val) 
			E.db.chatPanel.border.background_inset = val
			chatframe:UpdateChatBackground()
		end,
		get =function(info)
			return E.db.chatPanel.border.background_inset
		end,
	}
	
	
	local filterGUIs = {
		{
			addNew = {
				name = L['Name'],
				type = 'editbox',
				order = 3,
				width = 'full',
				set = function(info, value)
				
				end,
				get = function(info)
					return
				end
			},
			selecting = {
				name = L['Select'],
				type = 'dropdown',
				order = 4,
				width = 'full',
				values = function()
					local t = {}
					
					for k, v in pairs(	E.db.chatPanel.nameFilter) do					
						t[k] = k
					end
					
					return t
				end,
				set = function(info, value)
					filterSelectedToken = value
					
					E.db.chatPanel.nameFilter[value] = E.db.chatPanel.nameFilter[value] or { enable = true }
				end,
				get = function(info)
					return filterSelectedToken
				end,
			},
			opts = {
				name = L["Settings"],
				type = "group",
				order = 5,
				embend = true,
				args = {
					enable = {
						name = L['Enable'],
						type = 'toggle',
						order = 1, width = 'full',
						set = function()
							if filterSelectedToken then							
								if E.db.chatPanel.nameFilter[filterSelectedToken] then
									 E.db.chatPanel.nameFilter[filterSelectedToken].enable = not E.db.chatPanel.nameFilter[filterSelectedToken].enable
								end
							end
							
						end,
						get = function()
							if filterSelectedToken then							
								if E.db.chatPanel.nameFilter[filterSelectedToken] then
									return E.db.chatPanel.nameFilter[filterSelectedToken].enable
								end
							end
							return false
						end,	
					},						
				},	
			},
		},
		{
			addNew = nil,
			selecting = {
				name = L['Select'],
				type = 'dropdown',
				order = 4,
				width = 'full',
				values = function()
					local t = {}
					
					local totalBNet, numBNetOnline = BNGetNumFriends()
					
					for i=1, totalBNet do
						local presenceID, presenceName, battleTag, isBattleTagPresence, toonName, toonID, client, isOnline, 
							lastOnline, isAFK, isDND, messageText, noteText, isRIDFriend, broadcastTime, canSoR = BNGetFriendInfo(i)
						
						
						t[battleTag] = presenceName..' - '..battleTag.. ( noteText and ' - '..noteText or '')
					end
						
					return t
				end,
				set = function(info, value)
					filterSelectedToken = value			
					E.db.chatPanel.battleTagFilter[value] = E.db.chatPanel.battleTagFilter[value] or { enable = true }
				end,
				get = function(info)
					return filterSelectedToken
				end,
			},
			opts = {
				name = L["Settigns"],
				type = "group",
				order = 5,
				embend = true,
				args = {				
					enable = {
						name = L['Enable'],
						type = 'toggle',
						order = 1, width = 'full',
						set = function()
							if filterSelectedToken then							
								if E.db.chatPanel.battleTagFilter[filterSelectedToken] then
									 E.db.chatPanel.battleTagFilter[filterSelectedToken].enable = not E.db.chatPanel.battleTagFilter[filterSelectedToken].enable
								end
							end
							
						end,
						get = function()
							if filterSelectedToken then							
								if E.db.chatPanel.battleTagFilter[filterSelectedToken] then
									return E.db.chatPanel.battleTagFilter[filterSelectedToken].enable
								end
							end
							return false
						end,	
					},	

					inGame1 = {
						name = format(L['Game: |T%s:0:0|t%s'], BNet_GetClientTexture(BNET_CLIENT_WOW), 'World of Warcraft'),
						type = 'toggle',
						order = 2,
						set = function()
							if filterSelectedToken then							
								if E.db.chatPanel.battleTagFilter[filterSelectedToken] then
									 E.db.chatPanel.battleTagFilter[filterSelectedToken].game1 = not E.db.chatPanel.battleTagFilter[filterSelectedToken].game1
								end
							end
							
						end,
						get = function()
							if filterSelectedToken then							
								if E.db.chatPanel.battleTagFilter[filterSelectedToken] then
									return E.db.chatPanel.battleTagFilter[filterSelectedToken].game1
								end
							end
							return false
						end,	
					},
					inGame2 = {
						name = format(L['Game: |T%s:0:0|t%s'], BNet_GetClientTexture(BNET_CLIENT_D3), 'Diable 3'),
						type = 'toggle',
						order = 3,
						set = function()
							if filterSelectedToken then							
								if E.db.chatPanel.battleTagFilter[filterSelectedToken] then
									 E.db.chatPanel.battleTagFilter[filterSelectedToken].game2 = not E.db.chatPanel.battleTagFilter[filterSelectedToken].game2
								end
							end
							
						end,
						get = function()
							if filterSelectedToken then							
								if E.db.chatPanel.battleTagFilter[filterSelectedToken] then
									return E.db.chatPanel.battleTagFilter[filterSelectedToken].game2
								end
							end
							return false
						end,	
					},
					inGame3 = {
						name = format(L['Game: |T%s:0:0|t%s'], BNet_GetClientTexture(BNET_CLIENT_SC2), 'Starcraft 2'),
						type = 'toggle',
						order = 4,
						set = function()
							if filterSelectedToken then							
								if E.db.chatPanel.battleTagFilter[filterSelectedToken] then
									 E.db.chatPanel.battleTagFilter[filterSelectedToken].game3 = not E.db.chatPanel.battleTagFilter[filterSelectedToken].game3
								end
							end
							
						end,
						get = function()
							if filterSelectedToken then							
								if E.db.chatPanel.battleTagFilter[filterSelectedToken] then
									return E.db.chatPanel.battleTagFilter[filterSelectedToken].game3
								end
							end
							return false
						end,	
					},
					inGame4 = {
						name = format(L['Game: |T%s:0:0|t%s'], BNet_GetClientTexture(BNET_CLIENT_WTCG), 'Heartstone'),
						type = 'toggle',
						order = 5,
						set = function()
							if filterSelectedToken then							
								if E.db.chatPanel.battleTagFilter[filterSelectedToken] then
									 E.db.chatPanel.battleTagFilter[filterSelectedToken].game4 = not E.db.chatPanel.battleTagFilter[filterSelectedToken].game4
								end
							end
							
						end,
						get = function()
							if filterSelectedToken then							
								if E.db.chatPanel.battleTagFilter[filterSelectedToken] then
									return E.db.chatPanel.battleTagFilter[filterSelectedToken].game4
								end
							end
							return false
						end,	
					},
					inGame5 = {
						name = format(L['Game: |T%s:0:0|t%s'], BNet_GetClientTexture(BNET_CLIENT_OVERWATCH), 'Overwatch'),
						type = 'toggle',
						order = 6,
						set = function()
							if filterSelectedToken then							
								if E.db.chatPanel.battleTagFilter[filterSelectedToken] then
									 E.db.chatPanel.battleTagFilter[filterSelectedToken].game5 = not E.db.chatPanel.battleTagFilter[filterSelectedToken].game5
								end
							end
							
						end,
						get = function()
							if filterSelectedToken then							
								if E.db.chatPanel.battleTagFilter[filterSelectedToken] then
									return E.db.chatPanel.battleTagFilter[filterSelectedToken].game5
								end
							end
							return false
						end,	
					},
					inGame6 = {
						name = format(L['Game: |T%s:0:0|t%s'], BNet_GetClientTexture(BNET_CLIENT_HEROES), 'Heroest of the Storm'),
						type = 'toggle',
						order = 7,
						set = function()
							if filterSelectedToken then							
								if E.db.chatPanel.battleTagFilter[filterSelectedToken] then
									 E.db.chatPanel.battleTagFilter[filterSelectedToken].game6 = not E.db.chatPanel.battleTagFilter[filterSelectedToken].game6
								end
							end
							
						end,
						get = function()
							if filterSelectedToken then							
								if E.db.chatPanel.battleTagFilter[filterSelectedToken] then
									return E.db.chatPanel.battleTagFilter[filterSelectedToken].game6
								end
							end
							return false
						end,	
					},
					inGame7 = {
						name = format(L['Client: |T%s:0:0|tBattle.Net'], BNet_GetClientTexture()),
						type = 'toggle',
						order = 8,
						set = function()
							if filterSelectedToken then							
								if E.db.chatPanel.battleTagFilter[filterSelectedToken] then
									 E.db.chatPanel.battleTagFilter[filterSelectedToken].game7 = not E.db.chatPanel.battleTagFilter[filterSelectedToken].game7
								end
							end
							
						end,
						get = function()
							if filterSelectedToken then							
								if E.db.chatPanel.battleTagFilter[filterSelectedToken] then
									return E.db.chatPanel.battleTagFilter[filterSelectedToken].game7
								end
							end
							return false
						end,	
					},	
				},	
			},
		},
		{
			addNew = {
				name = L["Message"],
				type = 'editbox',
				order = 3,
				width = 'full',
				set = function(info, value)
					filterSelectedToken = value
					
					E.db.chatPanel.msgFilter[value] = E.db.chatPanel.msgFilter[value] or { enable = true }
					
					UpdateFilterMsgList()
				end,
				get = function(info)
					return
				end
			},
			selecting = {
				name = L['Select'],
				type = 'dropdown',
				order = 4,
				width = 'full',
				values = function()
					local t = {}
					
					for k, v in pairs(E.db.chatPanel.msgFilter) do					
						t[k] = k
					end
					
					return t
				end,
				set = function(info, value)
					filterSelectedToken = value					
					E.db.chatPanel.msgFilter[value] = E.db.chatPanel.msgFilter[value] or { enable = true }				
					UpdateFilterMsgList()
				end,
				get = function(info)
					return filterSelectedToken
				end,
			},
			opts = {
				name = L["Settings"],
				type = "group",
				order = 5,
				embend = true,
				args = {
					enable = {
						name = L['Enable'],
						type = 'toggle',
						order = 1, width = 'full',
						set = function()
							if filterSelectedToken then							
								if E.db.chatPanel.msgFilter[filterSelectedToken] then
									E.db.chatPanel.msgFilter[filterSelectedToken].enable = not E.db.chatPanel.msgFilter[filterSelectedToken].enable
								end
							end
							
							UpdateFilterMsgList()
						end,
						get = function()
							if filterSelectedToken then							
								if E.db.chatPanel.msgFilter[filterSelectedToken] then
									return E.db.chatPanel.msgFilter[filterSelectedToken].enable
								end
							end
							return false
						end,	
					},
				},	
			},
		},
	}
	
	function chatframe:UpdateFilterGUI(index)
		E.GUI.args.ChatPanel.args.Blacklist.args.filteraddNew = filterGUIs[index].addNew
		E.GUI.args.ChatPanel.args.Blacklist.args.filterselecting = filterGUIs[index].selecting
		E.GUI.args.ChatPanel.args.Blacklist.args.filterGUI = filterGUIs[index].opts
	end
	
	local lastScale = 0
	
	function chatframe:UpdateCombatLogChatSettings()
		if E.db.chatPanel.disable_chatlog then
			ChatFrame2Tab:SetScale(0.0001)
		else
			ChatFrame2Tab:SetScale(lastScale)
		end	
	end
	
	local function LoadChatSettings()
		
		--E.db.chatPanel.enable_background
		--E.db.chatPanel.disable_chatlog
		
		if E.db.chatPanel.enable_background then
			chatFrameBackground:Show()
		else
			chatFrameBackground:Hide()
		end
		
		lastScale = ChatFrame2Tab:GetScale()
		
		chatframe:UpdateCombatLogChatSettings()
		
		UpdateFilterMsgList()
		
		chatframe:UpdateChatBackground()	
	end
	
	E.UpdateChatSettings = LoadChatSettings
	E:OnInit2(LoadChatSettings)
end

do
	--[[ Start popup creation ]]--
	local frame = CreateFrame("Frame", nil, E.UIParent)

	frame:SetBackdrop({
			bgFile = [[Interface\Buttons\WHITE8x8]],
			edgeFile = [[Interface\Buttons\WHITE8x8]],
			edgeSize = 1,
			insets = {left = 0, right = 0, top = 0, bottom =0}
		})
	frame:SetBackdropColor(0 , 0 , 0 , 0.7) --цвет фона
	frame:SetBackdropBorderColor(0 , 0 , 0 , 1) --цвет фона
	
	frame:SetSize(650, 40)
	frame:SetPoint("CENTER", E.UIParent, "CENTER")
	frame:SetFrameStrata("DIALOG")
	frame:Hide()

	local editBox = CreateFrame("EditBox", nil, frame)
	editBox:SetFontObject(ChatFontNormal)
	editBox:SetSize(610, 40)
	editBox:SetPoint("LEFT", frame, "LEFT", 10, 0)
	local hide = function(f) f:GetParent():Hide() end
	editBox:SetScript("OnEscapePressed", hide)

	local close_ = CreateFrame("Button", nil, frame)
	close_:SetNormalTexture("Interface\\Buttons\\UI-Panel-MinimizeButton-Up")
	close_:SetPushedTexture("Interface\\Buttons\\UI-Panel-MinimizeButton-Down")
	close_:SetHighlightTexture("Interface\\Buttons\\UI-Panel-MinimizeButton-Highlight", "ADD")
	close_:SetSize(32, 32)
	close_:SetPoint("RIGHT", frame, "RIGHT", -5, 0)
	close_:SetScript("OnClick", hide)
	--[[ End popup creation ]]--

	-- Avoiding StaticPopup taints by making our own popup, rather that adding to the StaticPopup list
	function chatframe:Popup(text)
		editBox:SetText(text)
		editBox:HighlightText(0)
		editBox:GetParent():Show()
	end
end


do
	local f, db = chatframe

	local keep = 20 -- number of messages to log (could do 10000 too, in theory...)

	local events = {
	  "CHAT_MSG_BATTLEGROUND",
	  "CHAT_MSG_BATTLEGROUND_LEADER",
	  "CHAT_MSG_BN_WHISPER", -- battle.net whispers will show wrong names
	  "CHAT_MSG_BN_WHISPER_INFORM", -- battle.net whispers will show wrong names
	--  "CHAT_MSG_CHANNEL", -- all channel related talk (general, trade, defense, custom channels, e.g.)
	  "CHAT_MSG_EMOTE", -- only "/me text" messages, not /dance, /lol and such
	  "CHAT_MSG_GUILD",
	  "CHAT_MSG_GUILD_ACHIEVEMENT",
	  "CHAT_MSG_OFFICER",
	  "CHAT_MSG_PARTY",
	  "CHAT_MSG_PARTY_LEADER",
	  "CHAT_MSG_INSTANCE_CHAT_LEADER",
	  "CHAT_MSG_INSTANCE_CHAT",
	  "CHAT_MSG_RAID",
	  "CHAT_MSG_RAID_LEADER",
	  "CHAT_MSG_RAID_WARNING",
	  "CHAT_MSG_SAY",
	  "CHAT_MSG_WHISPER",
	  "CHAT_MSG_WHISPER_INFORM",
	  "CHAT_MSG_YELL",
	  "PLAYER_LOGIN", -- not a part of the chat messages logging, must be kept to show log at login
	}

	local playerFlag = "CHATHISTORYOWNMSGFLAG" -- unique flag that means that this was our message (must not change in between sessions or you have to delete the savedvariables file)

	_G["CHAT_FLAG_"..playerFlag] = "|TInterface\\GossipFrame\\GossipGossipIcon.blp:0:0:1:-2:0:0:0:0:0:0:0:0:0|t "

	local _ChatEdit_SetLastTellTarget = ChatEdit_SetLastTellTarget
	function ChatEdit_SetLastTellTarget(...)
	  if chatframe.silent then
		return
	  end
	  return _ChatEdit_SetLastTellTarget(...)
	end

	local function timestamp()
	  local a_2 = select(2, ("."):split(GetTime() or "0."..random(1, 999), 2)) or 0
	
	  return time().."."..a_2
	end
	
	local temp2 = {}
	local function printsorted()	
	  local temp, data = {}
	  for id, _ in pairs(db) do
		table.insert(temp, tonumber(id))
	  end
	  table.sort(temp, function(a, b)
		return a < b
	  end)
	
	  for i = 1, #temp do
		data = db[tostring(temp[i])]
		if type(data) == "table" then
		  ChatFrame_MessageEventHandler(DEFAULT_CHAT_FRAME, data[20], unpack(data))
		end
	  end
		DEFAULT_CHAT_FRAME:AddMessage("  ")
		DEFAULT_CHAT_FRAME:AddMessage("------------------")
		DEFAULT_CHAT_FRAME:AddMessage("  ")
	end

	local function cleanup()
	  local c, k = 0
	  for id, data in pairs(db) do
		c = c + 1
		if (not k) or k > data[21] then
		  k = data[21]
		end
	  end
	  if c > keep then
		db[k] = nil
	  end
	end
	
	local function LoadChatHistory()
		if not AleaUIDB then AleaUIDB = {} end		
		if not AleaUIDB["ChatHistory"] then AleaUIDB["ChatHistory"] = {} end
	
		db = AleaUIDB["ChatHistory"]
		
		chatframe:ToggleChatHistory()
	end

	local function OvEventFunction(self, event, ...)
		local temp = {...}
		if #temp > 0 then
		  temp[20] = event
		  temp[21] = timestamp()
		  if temp[2] == UnitName("player") then
			temp[6] = playerFlag
		  end
		  db[temp[21]] = temp
		  cleanup()
		end	
	end
	
	for _, event in pairs(events) do
	  chatframe[event] = OvEventFunction
	end
	
	function chatframe:ToggleChatHistory()
		if E.db.chatPanel.history then
			f.silent = 1
			printsorted()
			f.silent = nil	
			
			for _, event in pairs(events) do
				chatframe:RegisterEvent(event)
			end
		else
			for _, event in pairs(events) do
				chatframe:UnregisterEvent(event)
			end			
			wipe(db)
			wipe(AleaUIDB["ChatHistory"])
		end
	end
	
	E:OnInit2(LoadChatHistory)
end

do	
	local step = 0.08
	local select = select
	local pairs = pairs
	local isBubble = function(frame)
		if frame:GetName() then return end
		if not frame:GetRegions() then return end
		local region = frame:GetRegions()
		if not region.GetTexture then return end
		
		return ( region:GetTexture() == [[Interface\Tooltips\ChatBubble-Background]] or region:GetTexture() == [[Interface\Tooltips\ChatBubble-Background.blp]] )
	end
	
	local skinBubbles
	skinBubbles = function(frame, ...)
		if not frame.isTransparentBubble and isBubble(frame) then 
			for i=1, frame:GetNumRegions() do
				local region = select(i, frame:GetRegions())
				
				if region:GetObjectType() == "Texture" then
					region:SetTexture("")
				elseif region:GetObjectType() == "FontString" then
				--	local f, s = region:GetFont()
					region:SetFont(STANDARD_TEXT_FONT, 12, 'OUTLINE')
					
					frame.text = region
				end
			end
			
			frame:SetBackdrop({
			  bgFile = 'Interface\\Buttons\\WHITE8x8', 
			  edgeFile ='Interface\\Buttons\\WHITE8x8', 
			  tile = false, tileSize = 0, edgeSize = E.UIParent:GetScale(), 
			  insets = { left = 0, right = 0, top = 0, bottom = 0}
			})	
			
			frame:SetBackdropColor(0,0,0,0.8)
			frame:SetBackdropBorderColor(frame.text:GetTextColor())
			
			frame:HookScript("OnShow", function(self)
				self:SetBackdropBorderColor(self.text:GetTextColor())
			end)
			
			frame.isTransparentBubble = true
		end
		if (...) then 
			skinBubbles(...)
		end
	end

	local numChildren = -1
	local WorldFrame = WorldFrame

	local function LoadChatBubbles()
		bubbles.elapsed = 0
		bubbles:SetScript('OnUpdate', function(self, elapsed)
			self.elapsed = self.elapsed + elapsed
			if self.elapsed < 0.1 then return end
			local count = WorldFrame:GetNumChildren()
			if(count ~= numChildren) then
				numChildren = count
				skinBubbles(WorldFrame:GetChildren())
			end
		end)
	end
	
	E:OnInit2(LoadChatBubbles)
end